﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Remake
{
    public struct Option<T>
    {
        public bool HasValue { get; private set; }
        public T Value { get; private set; }
        public Option(T t)
        {
            HasValue = true;
            Value = t;
        }

        public void With(Action<T> action)
        {
            if (HasValue) action(Value);
        }
        public Option<U> Map<U>(Func<T, U> action)
        {
            return HasValue ? Option.Some(action(Value)) : Option.None<U>();
        }
        public Option<U> Map<U>(Func<T, Option<U>> action)
        {
            return HasValue ? action(Value) : Option.None<U>();
        }
    }

    public static class Option
    {
        public static Option<T> Some<T>(T t)
        {
            return new Option<T>(t);
        }
        public static Option<T> None<T>()
        {
            return new Option<T>();
        }
    }

    public static class Utils
    {
        public static Option<T> GetByPrefix<T>(IEnumerable<T> items, string prefix, Func<T, string> toString)
        {
            var matches = items.Where(i => toString(i).StartsWith(prefix)).ToArray();
            return matches.Length == 1 ? Option.Some(matches[0]) : Option.None<T>();
        }

        public static Option<T> GetByPrefix<T>(IEnumerable<T> items, string prefix)
        {
            return GetByPrefix(items, prefix, x => x.ToString());
        }
    }
}
